package org.jgum.traversal;

/**
 * Determines if and how redundancies should be eliminated from a list of categories generated by a traversal strategy of a linearization function.
 * This flag is useful when a traversal algorithm does not warranty that any category will be visited at most once.
 * @author sergioc
 *
 */
public enum RedundancyCheck {
	/**
	 * Keep the first redundant category, redundant categories coming afterwards are ignored.
	 */
	KEEP_FIRST, 
	
	/**
	 * Keep the last redundant category, previous redundant categories are ignored.
	 */
	KEEP_LAST, 
	
	/**
	 * Ignore redundancy checks. 
	 * This may improve performance and should be used if the traversal strategy does not generate redundant categories (e.g., in single-inheritance categorizations).
	 */
	IGNORE; 
}
